 clear;
 close all;
%
% genmolo2.m; v. 2.1
% Jay J. Ague, Yale University
% Compute molar rock compositions useful for pseudosections.
% Basic genform code converted from fortran 77 to matlab.
% Original fortran version 1.0, 26 Mar 98.
%
% Note that all elements of the string vectors elsym and oxsym
% are the same length.
%
 elsym=['Si  ';'Ti  ';'Al  ';'Fe3+';'Fe2+';...
        'Mg  ';'Mn  ';...
        'Ca  ';'Na  ';'K   ';...
        'C   ';'H   '];
%
 oxsym=['SiO2 ';'TiO2 ';'Al2O3';'Fe2O3';'FeO  ';...
        'MgO  ';'MnO  ';...
        'CaO  ';'Na2O ';'K2O  ';...
        'CO2  ';'H2O  '];
%
 molwtox=[60.084 79.865 101.960 159.687 71.844...
          40.304 70.937 ...
          56.077 61.979 94.196 ...
          44.009 18.015];
%
 atpropo=[2 2 3 3 1 ...
          1 1 ...
          1 1 1 ...
          2 1];
%
 catfac=[0.5 0.5 0.6666667 0.6666667 1 ... 
         1   1 ... 
         1   2   2 ...
         0.5 2];
%
 idebug=0;
 ibig=0;
 onum = 12; %Number of oxides.
%
 fprintf(1,'Program genmolo2 \n\n')
 outfil=input('Enter name of output script file: ','s');
 fid=fopen(outfil,'wt');
 fprintf(fid,'Program genmolo2 \n\n');
% diary(outfil); %diary not used---open file instead
%
 for i=1:5000
%
  if i>1
    iquit=input('Stop? (return=no, 1=stop): ');
    if iquit==1
      break;
    end
  end
%    
  sampno=input('Enter sample number: ','s');
%
  wtpct=0.0;
  wtt=0.0;
%
  oxygens=10.0; %Arbitrary.   
%
% Following code block is for interactive data input. The wt. percent
% value is input as a string (dum), then converted to a double. If the user
% hits return, matlab gives a NaN; the if statement detects the NaNs and
% zeros them out. This keeps matlab from bombing if the user hits return
% to enter a zero value.
%
  indata=1;
  while indata == 1
    for j=1:onum
      dum = input(sprintf('Enter wt. percent %s: ',oxsym(j,1:5)),'s');
      wtpc(j)=str2double(dum);	    	
      if isnan(wtpc(j)) == 1
        wtpc(j) = 0.0;
      end	
    end
    indata=input('Input OK? (return=yes,1=no): ');
  end    
%
  for j=1:onum
    wtt=wtt+wtpc(j);
  end
%
  fprintf(1,'\nChemistry, Sample: %s \n\n',sampno)
  fprintf(1,'Input Analysis \n')
  fprintf(fid,'\nChemistry, Sample: %s \n\n',sampno);
  fprintf(fid,'Input Analysis \n');
  for j=1:onum
    if wtpc(j) > 1e-6
      fprintf(1,'%s %7.3f\n',oxsym(j,1:5),wtpc(j))
      fprintf(fid,'%s %7.3f\n',oxsym(j,1:5),wtpc(j));
    end  
  end
  fprintf(1,'\nTotal %6.2f\n',wtt)
  fprintf(fid,'\nTotal %6.2f\n',wtt);
  wtpcf=0.0; %wtpc(18)*0.4211;
  wtpccl=0.0; %wtpc(19)*0.2256;
  wtpct=wtt - wtpcf - wtpccl;
%  fprintf(1,'Total %6.2f minus F, Cl equiv \n\n',wtpct)
%  fprintf(fid,'Total %6.2f minus F, Cl equiv \n\n',wtpct);
%
  pmol = wtpc ./ molwtox;
  atop = pmol .* atpropo;
%
  atopt=0.0;
  for j=1:onum
    atopt=atopt+atop(j);
  end
%
  d=oxygens/atopt;
  anio=d * atop .*catfac;
%
  fprintf(1,'\nMoles per %3.0f oxygens, Sample: %s \n\n',oxygens,sampno)
  fprintf(fid,'\nMoles per %3.0f oxygens, Sample: %s \n\n',oxygens,sampno);
  for j=1:onum
    if anio(j) > 1e-6
      fprintf(1,'%s %6.4f\n',elsym(j,1:4),anio(j))
      fprintf(fid,'%s %6.4f\n',elsym(j,1:4),anio(j));
    end  
  end
  fprintf(1,'\n')
  fprintf(fid,'\n');
%
% Call to f_mol2ox2 to do oxygen calculations and produce output suitable
% for pseudosection calculations. oansfo currently unused.
  [oansfo] = f_mol2ox2(fid,sampno,anio);
%
 end  %main loop for i=1:5000
%
 fclose(fid);
%
%%%%%%%%%%%%%
%
 function [m] = f_mol2ox2(fid,sampno,anio)
%
% Jay J. Ague, Yale University
% Convert moles from input analysis into various outputs useful for
% pseudosections including Fe2+ and Fe3+.
%
% Note that all elements of the string vectors elsym and oxsym
% are the same length.
%
 moles(1)=anio(1); %Si
 moles(2)=anio(2); %Ti
 moles(3)=anio(3); %Al
 moles(4)=anio(4); %Fe3
 moles(5)=anio(5); %Fe2
 moles(6)=anio(6); %Mg
 moles(7)=anio(7); %Mn
 moles(8)=anio(8); %Ca
 moles(9)=anio(9); %Na
 moles(10)=anio(10); %K
 moles(11)=anio(11); %C
 moles(12)=anio(12); %H
% 
 elsym=['Si  ';'Ti  ';'Al  ';'Fe3+';'Fe2+';'Mg  ';...
 'Mn  ';'Ca  ';...
 'Na  ';'K   ';'C   ';'H   '];
%
 oxsym=['SiO2 ';'TiO2 ';'Al2O3';'Fe2O3';'FeO  ';...
 'MgO  ';'MnO  ';'CaO  ';...
 'Na2O ';'K2O  ';'CO2  ';'H2O  '];
%
 oxcon=[1 1 2 2 1 1 1 1 2 2 1 2 ];
%
 elmolox=[2 2 1.5 1.5 1 1 1 1 0.5 0.5 2 0.5 ]; 
%
 molwtox=[60.084 79.865 101.960 159.687 71.844...
          40.304 70.937 ...
          56.077 61.979 94.196 ...
          44.009 18.015];
%
 idebug=0;
 ibig=0;
 onum = 12; %Number of oxides.
%
 oxel=2;
% 
  wtt=0.0;
  totmol=0.0;
  wttot=100.0;    
%
% Convert moles element to moles oxide if necessary.
%
  if oxel == 2
    for j=1:onum
      moles(j) = moles(j) / oxcon(j) ;
    end
  end
 %
  for j=1:onum
      wtpc(j) = moles(j) * molwtox(j);
  end
 
  for j=1:onum
    wtt=wtt+wtpc(j);
    totmol = totmol + moles(j);
  end
%
  for j=1:onum
    wtpc(j)=wttot*wtpc(j)/wtt;
  end
%
  fprintf(1,'\nBulk Chemistry, Sample: %s \n\n',sampno)
  fprintf(1,'Oxide wt percents based on 100 wtpct total \n')
  fprintf(fid,'\nBulk Chemistry, Sample: %s \n\n',sampno);
  fprintf(fid,'Oxide wt percents based on 100 wtpct total \n');
  for j=1:onum
    if wtpc(j) > 1e-6
      fprintf(1,'%s %7.3f\n',oxsym(j,1:5),wtpc(j))
      fprintf(fid,'%s %7.3f\n',oxsym(j,1:5),wtpc(j));
    end  
  end
 % fprintf(1,'\nTotal: %6.2f\n',wtt)
 % fprintf(fid,'\nTotal: %6.2f\n',wtt);
  fprintf(1,'wt percent total: %6.2f\n',wttot)
  fprintf(fid,'\nwt percent total: %6.2f\n',wttot);
%  
  fprintf(1,'\nBulk Chemistry, Sample: %s \n\n',sampno)
  fprintf(1,'Mole percent oxide\n')
  fprintf(fid,'\nBulk Chemistry, Sample: %s \n\n',sampno);
  fprintf(fid,'Mole percent oxide\n');
  for j=1:onum  
    molpct(j)=moles(j)*100.0/totmol;
    if wtpc(j) > 1e-6  
      fprintf(1,'%s %7.3f\n',oxsym(j,1:5),molpct(j))
      fprintf(fid,'%s %7.3f\n',oxsym(j,1:5),molpct(j));
    end
  end
%
  totfemol = molpct(4)*2+molpct(5);
  totofemol = molpct(4)*3 + molpct(5);
  odiff = totofemol - totfemol;
  totmolf = 100.0 - molpct(4) - molpct(5) + odiff + totfemol;
  f=100.0/totmolf;
  molpctf=f*molpct;
%  fprintf(1,'Moles Fe as FeO: %8.4f\n',totfemol)
%  fprintf(fid,'Moles Fe as FeO: %8.4f\n',totfemol);
%  fprintf(1,'Moles excess oxygen as O and O2: %8.4f %8.4f\n\n',odiff,odiff/2.)
%  fprintf(fid,'Moles excess oxygen as O and O2: %8.4f %8.4f\n\n',odiff,odiff/2.);
  fprintf(1,'\nMol percent with Fe as FeO + O\n')
  fprintf(1,'   SiO2    TiO2   Al2O3       O     FeO     MgO     MnO     CaO    Na2O     K2O     CO2     H2O\n')
  fprintf(fid,'\nMol percent with Fe as FeO + O\n');
  fprintf(fid,'   SiO2    TiO2   Al2O3       O     FeO     MgO     MnO     CaO    Na2O     K2O     CO2     H2O\n');
  fprintf(1,'%7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f\n\n',...
      molpctf(1),molpctf(2),molpctf(3),odiff*f,totfemol*f,molpctf(6),...
      molpctf(7),molpctf(8),molpctf(9),molpctf(10),molpctf(11),molpctf(12))
  fprintf(fid,'%7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f\n\n',...
      molpctf(1),molpctf(2),molpctf(3),odiff*f,totfemol*f,molpctf(6),...
      molpctf(7),molpctf(8),molpctf(9),molpctf(10),molpctf(11),molpctf(12));
%
  fprintf(1,'Moles element (not in %%)\n')  
  fprintf(fid,'Moles element (not in %%)\n');
  for j=1:onum  
    m(j)=molpct(j)*oxcon(j);
    if wtpc(j) > 1e-6    
      fprintf(1,'%s %7.3f\n',elsym(j,1:4),molpct(j)*oxcon(j))
      fprintf(fid,'%s %7.3f\n',elsym(j,1:4),molpct(j)*oxcon(j));
    end
  end
  oxtot = 0.0;
  for j=1:onum
      oxtot = oxtot + molpct(j) * oxcon(j) * elmolox(j);
  end
  fprintf(1,'Total moles oxygen (O): %8.4f\n',oxtot)
  fprintf(fid,'Total moles oxygen (O): %8.4f\n',oxtot);  
%  
  fprintf(1,'\n')
  fprintf(fid,'\n');
%
 fprintf(1,'SI(%7.4f)TI(%7.4f)AL(%7.4f)FE(%7.4f)FE(%7.4f)MG(%7.4f)MN(%7.4f)CA(%7.4f)NA(%7.4f)K(%7.4f)C(%7.4f)H(%7.4f)O(%8.4f)O(%7.4f)\n\n',...
     m(1),m(2),m(3),m(4),m(5),m(6),m(7),m(8),m(9),m(10),m(11),m(12),oxtot-odiff,odiff)
 fprintf(fid,'SI(%7.4f)TI(%7.4f)AL(%7.4f)FE(%7.4f)FE(%7.4f)MG(%7.4f)MN(%7.4f)CA(%7.4f)NA(%7.4f)K(%7.4f)C(%7.4f)H(%7.4f)O(%8.4f)O(%7.4f)\n',...
     m(1),m(2),m(3),m(4),m(5),m(6),m(7),m(8),m(9),m(10),m(11),m(12),oxtot-odiff,odiff);
%
% Project out Mn via spessartine.
 siprj = m(1) - 3.0*m(7); %Si
 alprj = m(3) - 2.0*m(7); %Al
 mnprj = 0.0; %Mn
 oprj = oxtot - 12.0*m(7); %O
%
 fprintf(1,'Mn projected out via spessartine\n')
 fprintf(fid,'\nMn projected out via spessartine\n');
 fprintf(1,'SI(%7.4f)TI(%7.4f)AL(%7.4f)FE(%7.4f)FE(%7.4f)MG(%7.4f)MN(%7.4f)CA(%7.4f)NA(%7.4f)K(%7.4f)C(%7.4f)H(%7.4f)O(%8.4f)O(%7.4f)\n\n',...
     siprj,m(2),alprj,m(4),m(5),m(6),mnprj,m(8),m(9),m(10),m(11),m(12),oprj-odiff,odiff)
 fprintf(fid,'SI(%7.4f)TI(%7.4f)AL(%7.4f)FE(%7.4f)FE(%7.4f)MG(%7.4f)MN(%7.4f)CA(%7.4f)NA(%7.4f)K(%7.4f)C(%7.4f)H(%7.4f)O(%8.4f)O(%7.4f)\n',...
     siprj,m(2),alprj,m(4),m(5),m(6),mnprj,m(8),m(9),m(10),m(11),m(12),oprj-odiff,odiff);
% 
 end